#!/usr/bin/perl

=head1 NAME
cree_obsoulpaobvent.pl

=head1 SYNOPSIS
(Taper la commande "perldoc -u cree_obsoulpaobvent.pl" pour obtenir la documentation de ce script)

* Lancer cree_obsoulpaobvent.pl sans argument

* Utilise un fichier d'entree nomme PAOBVENT issu d'une extraction oulan des PAOB.
* Produit :
-un fichier obsoul.paobvent contenant les PAOB de vent au format obsoul
-un fichier log_paobvent constituant le fichier log de l'execution de cree_obsoulpaobvent.pl

* Description du fichier PAOBVENT :
Il commence toujours par une ligne de la forme YYYYMMDD rr (comme l'OBSOUL) pour identifier le reseau extrait.
Suivent ensuite autant de lignes qu'il y a de PAOB extraits, chaque donnee (reel, entier ou chaine) etant separee par un espace. L'ordre des donnees en suivant la nomenclature IAA de la BDM est le suivant (selon les spec. recues) :

nom_tempete (char*8)
id_tempete (char*8)
centre_orig (int)
date observation (int) sous la forme YYYYMMDD
heure observation (int) sous la forme HHMMSS
latitude (real)
longitude (real)
amend (int)
id_terre_mer (real)
dd_syn_car (real)
ff_syn_car (real)
pmer1 (real)
raf_moyen (real)
ray_syn_car2 (real)
nb_seuil (real)
[
direc_ini (real)
direc_fin (real)
seuil_ff (real)
rayon_vent (real)
]

NB 1 : le bloc d'elements entre crochets est repete autant de fois que specifie par la valeur de nb_seuil.
NB 2 : les valeurs indeterminees entieres sont mises à la valeur NABSO (=999999), les reelles à RABSO(=1.7000E+38).

=head1 DESCRIPTION
Cree un vortex 3D de vent cyclonique selon le modele analytique de Holland (1980),
Sur deux couronnes à 100 et 200 km du centre du cyclone, aux niveaux 10m, 850, 700 et 500 hPa.
8 vecteurs vent sont produits par couronne et par niveau.
A partir de la Pmer au centre, du vent max à 10m et du rayon de vent max, sont produits les vents du vortex à 10m selon le modele de Holland. Un facteur d'attenuation avec l'altitude donne les vents sur les niveaux isobares superieurs.
Le flux directeur, approxime au vecteur deplacement du cyclone sur les 6 dernieres heures, est ajoute au vortex cyclonique symetrique de facon à prendre en compte les asymetries.
A noter qu'une convergence de 12 deg. est appliquee en plus au vent à 10m pour prendre en compte les frottements sur la surface.

* Ce script contient 3 modules :
-main : principal
-bogus : calcule les caracteristiques de bogus de vent (latitude,longitude,cap, force du vent) aux differents niveaux demandes.
-create_obs : a partir des sorties du module bogus, cree les fichiers obsoul et refdata correspondant.
-create_geo : cree des bogus au format geopoint de metview afin de pouvoir visualiser les bogus. Ce module sert essentiellement de verification et n'est pas appele en operationnel.

* Modifications apportees en mars 2011 pour integration aux Aladins-OM
- autorisation du bogus dans l'Hemisphere Nord
- si Atlantique Nord, on applique le coefficient 0.93 au Vmax (conversion necessaire car le CMRS de Miami produit du vent moyen sur 1min au lieu de 10min)
- test si le centre du cyclone est a plus de 1 degres a l'interieur du domaine Aladin, pour eviter un bogus partiel ; les coordonnees des domaines sont entrees en dur dans le script
- verifie qu'il n'existe qu'un seul bogus pour un meme systeme a un instant (probleme apparu dans le Pacifique Sud, CMRS de Nadi)
- changement du nom du fichier d'entrée : PAOBREU -> PAOBVENT
- phasage sur la version du script oper (vs expé Olive) : changement de la densité de l'air de 1.29 à 1.15, et activation de l'atténuation des vents selon la hauteur

* Modification mai 2012 :
- nettoyage (sortie plus propre en cas d'absence de fichier d'entree)

* Modification septembre 2012 :
- ajout du rayon de vent max + 10 km s'il est inferieur ou egal a 60km

=head1 AUTHOR

    Samuel Westrelin (DIRRE/CRC) - Juillet 2007
    Matthieu Plu, Ghislain Faure (DIRRE/CRC) - Mars 2011
    Dominique Mkis (DIRRE/CRC) - Mai 2012
    Matthieu Plu (DIRRE/CRC) - Septembre 2012
=cut


#-----------------------------------------------------------------------------------

package create_geo ;

  use strict;
  use Shell qw (mkdir);

  sub new {
     my $class = shift;
     return bless { @_ }, $class;
  }

   sub date {
      my $self = shift;
      return $self->{date};
   }

   sub hhmm {
      my $self = shift;
      return $self->{hhmm};
   }

   sub level {
      my $self = shift;
      return $self->{level};
   }

   sub boguslist {
      my $self = shift;
      return $self->{boguslist};
   }

   sub num_cyc {
      my $self = shift;
      return $self->{num_cyc};
   }

   sub geopoint {
#    Creation de fichiers geopoints pour visualisation sous metview
     my $self = shift;

     my $level = $self->level();
     my $date  = $self->date();
     my $hhmm  = $self->hhmm();
     my $boguslist = $self->boguslist();
     my $num_cyc = $self->num_cyc();
     my $geodir = "${num_cyc}/GEOPOINTS" ;
     ( -d $geodir ) || mkdir ("-p",$geodir);

     for my $coord ("u","v") {
       my $filegeo = "${geodir}/${coord}_bogus${date}${hhmm}_${level}hPa.geo";
       open FILE,">$filegeo" ;
       print FILE <<"EOF";
#GEO
#DATA LAT LON LEVEL YYYYMMDD HHMM VAL
EOF

#print Data::Dumper::Dumper(@$boguslist) ;

       for my $elt ( @$boguslist ) {
         printf FILE ("%5.1f %5.1f %5d %8d %4d %5.1f\n",
                      $elt->{lat},$elt->{lon},$level,$date,$hhmm,$elt->{$coord});
       }
       close FILE;
     }
   }

1; # Fin package create_geo

#-----------------------------------------------------------------------------------

package create_obs ;

  use strict;
  use Shell qw (mkdir);

  sub new {
     my $class = shift;
     return bless { @_ }, $class;
  }

   sub date {
      my $self = shift;
      return $self->{date};
   }

   sub hms {
      my $self = shift;
      return $self->{hms};
   }

   sub center {
      my $self = shift;
      return $self->{center};
   }

   sub num_cyc {
      my $self = shift;
      return $self->{num_cyc};
   }

   sub boguslist {
      my $self = shift;
      return $self->{boguslist};
   }

   sub compteur {
      my $self = shift;
      return $self->{compteur};
   }

   sub compteur_general {
      my $self = shift;
      return $self->{compteur_general};
   }

   sub outputfiles {
#    Creation des fichiers obsoul.paobvent et refdata.paobvent
      my $self = shift;
      my $date = $self->date();
      my $hms = $self->hms();
      my $h = $hms/10000;
      my $hh = sprintf "%2.2d",$h ;
      my $nbogus  = $self->compteur_general();

      my $obsoulfile ;
      if ( $nbogus == 0 ) {
        open $obsoulfile, "> obsoul.paobvent" ;
        printf $obsoulfile (" %8d %d\n",$date,$h);

        open REFDATAFILE,">refdata.paobvent" ;
        printf REFDATAFILE ("%8s %8s %16s %8d %2s\n",'paobvent','OBSOUL  ','paobvent        ',$date,$hh);
        close REFDATAFILE ;

      } else {
        open $obsoulfile, ">> obsoul.paobvent" ;
      }

      return $obsoulfile;
   }


   sub param {
#    Choix du parametre vent a 10m ou vent d'altitude
#    pour ecriture dans fichier obsoul
     my ($class,$level) = @_;

     my $param;
     if ( $level > 95000 ) {
        $param = 41;
     } else {
        $param = 3;
     }
     return $param;
   }


   sub observation {
#    Creation de fichier d'observation au format OBSOUL comme
#    fichier d'entree dans batodb puis assimilation
     my $self = shift;

     my $date  = $self->date();
     my $hms  = $self->hms();
     my $h = $hms/10000;
     my $inum  = $self->compteur();
     my $num_cyc = sprintf "%3s",$self->num_cyc();
     my $center= sprintf "%3s",$self->center();
     my $boguslist = $self->boguslist();
     my $obsoulfile = $self->outputfiles();

     my $nparam = scalar @$boguslist;
     my $i = 12+$nparam*5;
     my $alt="0.0";

     my $ic = 0;
     for my $elt ( @$boguslist ) {
       $ic+=1;
       $inum = sprintf "%2.2d",$inum;
       my $id = "'${num_cyc}${inum}${center}'";
       my $level = $elt->{level};
       my $param = $self->param($level);
       if ( $ic == 1 ) {
          printf $obsoulfile (" $i 8 181  %.5f   %.5f  %10s  %8d %6d $alt %d 11100 3 ",
                            $elt->{lat},$elt->{lon},$id,$date,$hms,$nparam);
       }
       printf $obsoulfile (" %d %.1f %.1f %.1f 2560 ",$param,$level,$elt->{vit},$elt->{cap} );
       if ( $ic == $nparam ) {
          printf $obsoulfile ("\n");
       }
     }

     close $obsoulfile ;
   }

1; # Fin package create_obs

#-----------------------------------------------------------------------------------


package bogus;

   use strict;
#  use Math::FixedPrecision;
   use POSIX qw(acos asin atan);

   our $degrad ;
   $degrad = atan2(1,1)*4/180. ;


   sub new {
      my $class = shift;
      return bless { @_ }, $class;
   }

   sub clat {
      my $self = shift;
      return $self->{clat};
   }

   sub clon {
      my $self = shift;
      return $self->{clon};
   }

   sub dangle {
      my $self = shift;
      return $self->{dangle};
   }

   sub level {
      my $self = shift;
      return $self->{level};
   }

   sub pc {
      my $self = shift;
      return $self->{pc};
   }

   sub rm {
      my $self = shift;
      return $self->{rm};
   }

   sub vm {
      my $self = shift;
      return $self->{vm};
   }

   sub r {
      my $self = shift;
      return $self->{r};
   }

   sub cap {
      my $self = shift;
      return $self->{cap};
   }

   sub vit {
      my $self = shift;
      return $self->{vit};
   }

   sub hemisphere {
      my $self = shift;
      return $self->{hemisphere};
   }


   sub vortex {
#  Restitue un vortex de Holland
     my $self = shift;

     my $dangle = $self->dangle();
     my $hemisphere = $self->hemisphere();

#    Coordonnees (lat,lon) du point de coord polaire (r,alpha)
     my ($lat,$lon) = $self->dpe_formula ( $dangle ) ;

#    Calcul du vecteur deplacement du cyclone sur les dernieres heures
     my ($uc,$vc) = $self->u_v_cyclone();

#    on ne retourne pas de bogus qui franchirait l'equateur !!
     if ( $lat*$hemisphere > 0 ){
#      Calcul du vecteur vent du vortex cyclonique
#      Il faut ajouter -90degres a dangle pour avoir l'angle entre le zero
#      trigonometrique et ce vecteur vent puisqu'il s'agit du vent tangent 
#      au cercle cyclonique
#      !!!!! Hem N : $dangle+90 ; Hem S : $dangle-90 !!!!!
       my $ffh = $self->holland_vent_tangentiel();
       my ($uh,$vh) = $self->bogus_vent( $dangle+90*$hemisphere, $ffh );

#      Ajout du vecteur deplacement du cyclone sur les dernieres heures
#      pour creer les asymetries
       if ( defined $uc && defined $vc ) {
         $uh+=$uc;
         $vh+=$vc;
       }

#      Calcul des cap, vitesse, position
       my $position = $self->bogus_position($uh,$vh) ;

       return $position ;

     }
   }


   sub bogus_position {
     my ($class, $u, $v) = @_ ;

     my $dangle = $class->dangle();
     my $level  = $class->redefine_level();

#    Coordonnees (lat,lon) du point de coord polaire (r,alpha)
     my ($lat,$lon) = $class->dpe_formula ( $dangle ) ;

     my @bogus = () ;
#    Calcul des cap et vitesse a partir des composantes du vent
     my ($cap, $force) = $class->cylindric_coord ($u, $v);
#    ($cap, $force) = map { Math::FixedPrecision->new($_,1) } ($cap, $force);
     push @bogus,{'lat' => $lat,'lon' => $lon,'level'=>$level,
                  'cap'=>$cap,'vit'=>$force,'u'=>$u,'v'=>$v};

     return \@bogus ;
   }


   sub redefine_level {
#    redefinit le niveau "level" 
#    seul le niveau 1000hPa est ajuste a une valeur deduite de la formule de Holland
#    les autres niveaux restent inchanges
     my $self = shift;
     my $level = $self->level();

     if ( $level == 100000 ) {
       $level = $self->holland_formula();
     }
     return $level;
   }


   sub holland_formula {
#
#    Modele analytique de cyclone (Holland, 1980)
#    Calcule la pression mer a une distance r du centre
#    du cyclone a partir de la pression au centre, de la pression environnementale,
#    du rayon de vent max et du vent max.
#
     my $self = shift;

     my $rm = $self->rm();
     my $r  = $self->r();
     my $vm = $self->vm();
     my $pc = $self->pc();

#    pc = Pmer au centre du cyclone
#    rm = rayon de vent max
#    vm = vent max
#    r  = distance au centre a laquelle on va estimer la pression mer

     my $pe = 101328.0;   # Pmer environnement fixee a 1013.28hPa
     my $rho = 1.15 ;   # Densite de l'air

     my $b = $rho*exp(1)*($vm**2)/($pe - $pc) ;

     my $p = $pc + ($pe - $pc)*exp(-($rm/$r)**$b);

     return $p ;
   }


   sub inflow {
#    Restitue le vent rentrant de 12 degres en surface et nul ailleurs
#    pour la construction du vortex cyclonique
     my $self = shift;

     my $level = $self->level();
     my $hemisphere = $self->hemisphere();

     my $inflowangle ;
#    !!!!! Hem N : $inflowangle=-12 ; Hem S : $inflowangle=12 !!!!!
     if ( $level == 100000 ) {
       $inflowangle = -12*$hemisphere;
     } else {
       $inflowangle = 0;
     }

     return $inflowangle;
   }


   sub bogus_vent {
#    Calcule les composantes du vent a partir de la force du vent et
#    de l'angle par rapport au zero trigonometrique
     my ( $class, $dangle, $ff ) = @_;

     my $inflow = $class->inflow();
     ( $dangle, $inflow ) = map { $_ * $degrad } ( $dangle, $inflow );
     my $u = $ff * cos($dangle - $inflow );
     my $v = $ff * sin($dangle - $inflow );
 
     return ($u,$v);
   }

   sub cylindric_coord {
#    Calcule cap et vitesse a partir des composantes du vent
     my ( $class, $u, $v ) = @_;

     my $ff = sqrt($u**2 + $v**2);
     my $angle = acos($u/$ff);
     $angle=$angle/$degrad;
     my $cap ;
#    calcul du cap (vent du nord cap=360degres ; vent d'est cap=90degres)
     if ( $v >= 0 ) {
       $cap = 270-$angle;
     } elsif ( $u>=0 && $v<0 ) {
       $cap = 270+$angle;
     } else {
       $cap = $angle-90;
     }
     if ( $cap == 0 ) {
       $cap = 360;
     }
#    print "Cylindrique $u $v $angle $cap $ff \n";

     return ($cap, $ff);
   }


   sub u_v_cyclone {
#    Calcule les composantes horizontales du vecteur deplacement du cyclone
#    sur les dernieres heures a partir de cap et vitesse
     my $self = shift;

     my $cap = $self->cap();
     my $vit = $self->vit();
     my $u;
     my $v;

     if ( defined $cap && defined $vit ) {
       my $angle = (90-$cap)*$degrad;
       $u = $vit*cos($angle); 
       $v = $vit*sin($angle); 
     } else {
       $u = undef;
       $v = undef;
     }

     return ($u,$v);

   }


   sub holland_vent_tangentiel {
#    Calcule la force du vent tangentiel cyclonique a une distance r du centre
#    avec le modele analytique de Holland
     my $self = shift;

     my $rm = $self->rm();
     my $r  = $self->r();
     my $vm = $self->vm();
     my $pc = $self->pc();
     my $level = $self->level();
     my $pe = 101328.0;   # Pmer environnement fixee a 1013.28hPa
     my $rho = 1.15 ;   # Densite de l'air

#    Facteur d'attenuation de la force du vent selon l'altitude isobare en Pa
     my %levelfactor = (
          100000 => 1,
          85000  => 1,
          70000  => 0.95,
          50000  => 0.85,
          40000  => 0.65,
         );

     my $b = $rho*exp(1)*($vm**2)/($pe - $pc) ;
     my $f = $self->coriolis() ;

     my $a = exp($b*log($rm/$r)) ;
     my $c = $r*$r*$f*$f/4 ;

     my $ff = sqrt( $a*($vm**2)*exp(1-$a)+$c ) - sqrt($c) ; 

#    Attenuation du vent selon le niveau
     $ff = $ff * $levelfactor{$level};

     return $ff ;
  }


  sub coriolis {
# Calcule le parametre de Coriolis en un point
     my $self = shift;

     my $dangle = $self->dangle();

#    Coordonnees (lat,lon) du point de coord polaire (r,alpha)
     my ($lat,$lon) = $self->dpe_formula ( $dangle ) ;

     my $omega = 0.0000729 ; #Vitesse de rotation de la Terre : 7.29*10-5 radian/s

     my $f = 2*$omega*sin($lat*$degrad) ;
     
     return $f ;
  }


  sub dpe_formula {
#
#   On considere le cyclone comme un cercle trigonometrique de rayon r
#   pour determiner les lat et lon d'un point du cercle situe a un angle
#   alpha de l'angle 0.
#   Le centre du cyclone est de coordonnees (clon, clat)
#

    my ( $class, $alpha  ) = @_;
    my $clat = $class->clat();
    my $clon = $class->clon();
    my $r = $class->r();

#   $r : Distance (en m) des points au centre du cyclone
#   Coordonnees converties en radians
    ( $clon, $clat, $alpha ) = map { $_ * $degrad } ( $clon, $clat, $alpha ) ;

#   Rayon terrestre (en m)
    my $R = 6371229 ;

    my $a = sin($alpha) ;
    my $b = cos($alpha) ;
    my $c = $r/$R ;

#   Coordonnees du point recherche
    my $lon = $c*$b + $clon ;
    my $lat = $c*$a + $clat ;

#   Conversion en degres
#   ( $lon, $lat ) = map { Math::FixedPrecision->new($_ / $degrad,1) } ( $lon, $lat ) ;
    ( $lon, $lat ) = map { $_ / $degrad } ( $lon, $lat ) ;

    return ($lat,$lon);

  }

1; # Fin package bogus

#-----------------------------------------------------------------------------------

package main;

  use strict;

#Definition des domaines des Aladins - Ce sera utilise plus loin pour ne pas produire de bogus si le centre du systeme est trop pres des bords des domaines
#Format : (lat_min,lat_max,lon_min,lon_max)
my @dom_reunion = (-32.9,0.1,31.3,88.7);
#Domaines a affiner (grille de calcul)
my @dom_antig = (-2,28.7,-75.7,-44.3);
my @dom_ncal = (-31.2,-7.9,155.1,174.9);
my @dom_polyf = (-32,0.2,-165,-127.1);
#Limite (en degres) acceptable du centre du bogus par rapport aux bords 
my $lb = 1;

# Niveaux verticaux auxquels est produit un bogus de vent
my @levels = (100000,85000,70000,50000);

# Ouverture des fichiers d'entree PAOBVENT contenant les caracteristiques des cyclones necessaires au calcul du bogus de vent, fichier log log_paobvent, et fichier au format obsoul contenant les bogus de vent obsoul.paobvent
open LOGFILE, "> log_paobvent";

#--------------------------------------------------------------------
# Ouverture du fichier PAOBVENT en capturant les eventuelles erreurs
#--------------------------------------------------------------------

eval {
  open INPUTFILE, "< PAOBVENT" or die printf LOGFILE ("PAS DE FICHIER d'entree PAOBVENT \n");
};
if ($@) {
  print "STOP - Pas de fichier PAOBVENT\n";
  exit 0;
}


# Initialisation du nombre de bogus (un bogus etant considere comme le vent specifie aux 4 niveaux)
my $compteur_general = 0;

# Initialisation des systemes deja traites (pour tester la redondance eventuelle)
my @compteur_cyc = ();

# Lecture du fichier $inputfile
# Lecture de la premiere ligne
my (undef) = split /\s+/, <INPUTFILE> ;
# Lecture de la suite du fichier
while ( <INPUTFILE> ) {

    my ( undef,$cyc_name,undef,$num_cyc,$numcenter,$ymd,$hms,$lat,$lon,$amend,$id_terre_mer,$cap,$vit,$pmer,$vmax,$rmax,$nb_seuil,@ext_vent ) = split /\s*'\s*|\s+/;

    printf LOGFILE "****** cyc_name : $cyc_name ; num_cyc : $num_cyc *******\n";
    printf LOGFILE "numcenter : $numcenter,ymd : $ymd,hms : $hms,lat : $lat,lon : $lon,amend : $amend,id_terre_mer : $id_terre_mer,cap : $cap,vit : $vit,pmer : $pmer,vmax : $vmax,rmax : $rmax \n";
    printf LOGFILE "nb seuils vent : $nb_seuil,extensions de vent:@ext_vent \n\n" ; 

#   Pretraitements
#   Pas de bogus de vent produits pour l'hemisphere nord - Condition retiree
#    if ( $lat > 0 ) { printf LOGFILE "PAS DE BOGUS de VENT CALCULE pour l'Hemisphere NORD. \n\n";next};

#   Test si cyclone deja traite

    if ( "/@compteur_cyc/" =~ "$num_cyc" ) { printf LOGFILE $num_cyc; printf LOGFILE " est redondant - On ne garde pas ce bogus\n\n";next };

#   Cas du cyclone stationnaire
    if ( $cap == 0 ) {
      $vit=0;
    }
#   Cas ou cap manquant
    if ( $cap > 360 ) {
      $vit=undef;
    }
#   Cas ou vit manquante
    if ( $vit > 40 ) {
      $vit=undef;
    }

#   Condition sur ymd au cas ou elle serait manquante
    if ( $ymd == 999999 ) { printf LOGFILE "PAS DE BOGUS CALCULE car YYYYMMDD non renseignee !!!\n\n";next };

#   Condition sur hms au cas ou elle serait manquante
    if ( $hms == 999999 ) { printf LOGFILE "PAS DE BOGUS CALCULE car HHMMSS non renseignee !!!\n\n";next };

#   Formats date et heure
    my $h = $hms/10000 ;
    my $hhmm = sprintf "%2.2d00",$h ;

# Distances au centre auxquelles on calcule les vents cycloniques
#(100km, 200km, et rayonventmax+10km si inferieur ou egal a 60km)
    my @radiuslist ;
    if ( $rmax>60001 ) { @radiuslist = ( 100000, 200000 ) }
    else { @radiuslist = ( $rmax + 10000, 100000, 200000 ) }
    print LOGFILE "Rayons du bogus (m) : @radiuslist \n" ;

# Increments d'angle a ajouter a l'angle zero du cercle trigonometrique
# (en considerant le cyclone comme un cercle trigonometrique)
# pour obtenir la position des points de bogus
   my %radiuslatlon;
   foreach (@radiuslist) {
    if ($_>50000) { $radiuslatlon{"$_"} = [90,-90,0,180,45,-45,135,-135] }
    else { $radiuslatlon{"$_"} = [90,-90,0,180] }
   }

#   Condition sur rmax qui n'est parfois pas rentre dans la BDM (valeur enorme dans ce cas)
    if ( $rmax > 500000 ) { printf LOGFILE "!!! Rayon de vent max non renseigne : il est force a 45000!!!\n\n";$rmax=45000};

#   Condition sur vmax qui n'est parfois pas rentre dans la BDM (valeur enorme dans ce cas)
    if ( $vmax > 500 ) { printf LOGFILE "PAS DE BOGUS CALCULE car vent max non renseigne !!!\n\n";next };

#   Condition sur pmer au cas ou elle serait manquante
    if ( $pmer > 110000 ) { printf LOGFILE "PAS DE BOGUS CALCULE car pmer non renseignee !!!\n\n";next };

#   Condition sur lat au cas ou elle serait manquante
    if ( $lat > 90 ) { printf LOGFILE "PAS DE BOGUS CALCULE car latitude non renseignee !!!\n\n";next };

#   Condition sur lon au cas ou elle serait manquante
    if ( $lon > 360 ) { printf LOGFILE "PAS DE BOGUS CALCULE car longitude non renseignee !!!\n\n";next };

#   Indice indiquant l'hemisphere (hemisphere=1 pour HN, -1 pour HS)
    my $hemisphere=-1 ;
    if ( $lat > 0 ) { $hemisphere=1 };

#   Test de la proximite aux bords des domaines Aladin
    my $in_domain = 0;
    if ( ( $lat > $dom_reunion[0] + $lb ) and ( $lat < $dom_reunion[1] - $lb ) and ( $lon > $dom_reunion[2] +$lb ) and ( $lon < $dom_reunion[3] - $lb ) ) { printf LOGFILE "Cyclone a l interieur du Domaine Aladin Reunion \n";$in_domain = 1 };
    if ( ( $lat > $dom_antig[0] + $lb ) and ( $lat < $dom_antig[1] - $lb ) and ( $lon > $dom_antig[2] +$lb ) and ( $lon < $dom_antig[3] - $lb ) ) { printf LOGFILE "Cyclone a l interieur du Domaine Aladin Antilles-Guyane \n";$in_domain = 1 };
    if ( ( $lat > $dom_ncal[0] + $lb ) and ( $lat < $dom_ncal[1] - $lb ) and ( $lon > $dom_ncal[2] +$lb ) and ( $lon < $dom_ncal[3] - $lb ) ) { printf LOGFILE "Cyclone a l interieur du Domaine Aladin Nouvelle Caledonie \n";$in_domain = 1 };
    if ( ( $lat > $dom_polyf[0] + $lb ) and ( $lat < $dom_polyf[1] - $lb ) and ( $lon > $dom_polyf[2] +$lb ) and ( $lon < $dom_polyf[3] - $lb ) ) { printf LOGFILE "Cyclone a l interieur du Domaine Aladin Polynesie \n";$in_domain = 1 };

   if ( $in_domain == 0 ) { printf LOGFILE "Cyclone trop pres des bords !\n";next };

#   Detection du bassin et application de la conversion VentMax si Atlantique
    if ( $num_cyc =~ "L" ) { printf LOGFILE "!!! Atlantique Nord : conversion VentMax facteur 0.93 - vmax = ";$vmax=0.93*$vmax;printf LOGFILE $vmax };

#   Cyclone accepte : on le met dans le compteur
    #push @compteur_cyc, $num_cyc;
    @compteur_cyc = (@compteur_cyc , $num_cyc);

#   Initialisation du tableau associatif contenant tous les bogus de vent
    my %allposition = ();


#   Construction des bogus de vent
#   Boucles sur les niveaux, rayons, angles
    for my $level ( @levels ) {
      my @position = ();
      for my $r ( @radiuslist ) {
      for my $dangle ( @{$radiuslatlon{$r}} ) {
         my $bogus = bogus->new(
              clat => $lat,
              clon => $lon,
              dangle => $dangle,
              level  => $level,
              r    => $r,
              rm   => $rmax,
              vm   => $vmax,
              cap  => $cap,
              vit  => $vit,
              pc  => $pmer,
              hemisphere  => $hemisphere,
              );
         my $newposition = $bogus->vortex();
         if ( scalar @$newposition > 0 ) {
           push @position, @$newposition;
           push @{$allposition{$r}{$dangle}}, @$newposition;
         }
      } # fin boucle sur les angles
      } # fin boucle sur les rayons

#=pod
# A activer pour creer les geopoints correspondants
      my $geopoint = create_geo->new(
              boguslist => \@position,
              level  => $level,
              date => $ymd,
              hhmm => $hhmm,
              num_cyc   => $num_cyc,
              );
      $geopoint->geopoint();
#=cut
    } # fin boucle sur les niveaux

#   Pour chaque position (lat lon identiques ce qui revient a r et dangle identiques)
#   creation de l'obs dans le fichier obsoul
    my $compteur = 1;
    for my $r ( @radiuslist ) {
    for my $dangle ( @{$radiuslatlon{$r}} ) {
      if (scalar @{$allposition{$r}{$dangle}} > 0 ) {
        my $windpaob = create_obs->new(
              boguslist => $allposition{$r}{$dangle},
              center    => $numcenter,
              num_cyc   => $num_cyc,
              date      => $ymd,
              hms      => $hms,
              compteur  => $compteur,
              compteur_general  => $compteur_general,
              );
        $windpaob->observation();
        $compteur+=1;
        $compteur_general+=1;
      }
    }
    }

} #Fin du while=fin lecture fichier d'entree
close INPUTFILE;
close LOGFILE;


#1; # Fin package main
#-----------------------------------------------------------------------------------
